import os

import numpy as np

import iris
import iris.cube
import iris.analysis

import warnings
import datetime

import numba
from numba import vectorize
from numba import cuda
from numba import njit
from numba import int32, int64, float32, float64
from numba.cuda.cudadrv import driver

import diagonals

__all__ = ['compute_zonmean', 'get_basin_area']


def compute_zonmean(var, lats, area):
    """Function that checks device and calls computing functions.

    Checks if the computations are going performed in the CPU or the GPU:

    Parameters
    ----------
    var : float32
        Masked array containing variable data.
    basins : float32
        List containing basin names and masks.
    area : float32
        Masked array containing cell area.

    Returns
    -------
    regmean: float32
        List containing regional mean for variable var
    """
    if diagonals.CONFIG.use_gpu:
        logger.warning('GPU routines not implemented for regmean diagnostic.'
                       'Using CPU instead')
    else:
        zonmean = _compute_zonmean_cpu(var, lats, area)
    return zonmean


def _compute_zonal_mean_cpu(var, lats, area):
    times = var.shape[0]
    levs = var.shape[1]
    value = {}
    for basin, mask in weight.items():
        for time in times:
            for lev in levs:
                value[basin] = _zonal_mean_cpu(var[t, l, :, :],
                                               area[basin],
                                               lats)
    return value


def _compute_zonal_mean_gpu(var, lats, area):
    times = var.shape[0]
    levs = var.shape[1]
    lats = var.shape[2]
    lons = var.shape[3]
    value = {}

    block = (128, 1, 1)
    grid_size = (lons // block[0]) + 1
    grid = (grid_size, lats)
    gpu_var = cuda.to_device(var.astype(np.float32))
    total = cuda.device_array(180, dtype=np.float32)
    weights = cuda.device_array(180, dtype=np.float32)
    for basin, mask in area.items():
        gpu_area = cuda.to_device(area[basin].astype(np.float32))
        gpu_value = _zonal_mean_gpu(gpu_var, gpu_area, lats)
        value[basin] = gpu_value.copy_to_host()
    del gpu_area, gpu_value, gpu_var

    return value


@numba.njit()
def _zonal_mean_cpu(variable, weight, latitude):
    total = np.zeros(180, np.float64)
    weights = np.zeros(180, np.float64)
    for i in range(variable.shape[0]):
        for j in range(variable.shape[1]):
            if weight[i, j] == 0:
                continue
            bin_value = int(round(latitude[i, j]) + 90)
            weights[bin_value] += weight[i, j]
            total[bin_value] += variable[i, j] * weight[i, j]
    return total / weights


@cuda.jit()
def _zonal_mean_gpu(variable, weight, latitude):
    total[i, j] = 0.0
    weights[i, j] = 0.0
    if(i >= total.shape[0]):
        return
    if(weight[i, j] != 0.0):
        bin_value = int(round(latitude[i, j]) + 90)
        weights[bin_value] += weight[i, j]
        total[bin_value] += variable[i, j] * weight[i, j]
    return total / weights


def get_basin_area(areacello, basins):
    basin_areas = {}
    for basin in basins:
        basin_areas[basin] = _compute_basin_area(areacello, basins[basin])
    return basin_areas


@vectorize(['float32(float32, float32)'], target='cpu')
def _compute_basin_area(areacello, basin):
    """Vectorized numba function executed on the cpu that computes the area
       for each basin:

    Parameters
    ----------
    areacello : float32
        Masked array containing areacello.
    basin: float32
        Masked array containing the mask for a given basin.

    Returns
    -------
    areacello*basin : float32
        Masked array containing the area for a given basin.
    """
    return areacello*basin
